﻿(function() {
    'use strict';

    /**
     * @ngdoc module
     * @name tubular.services
     * 
     * @description
     * Tubular Services module. 
     * It contains common services like Http and OData clients, and filtering and printing services.
     */
    angular.module('tubular.services', ['ui.bootstrap'])
        /**
         * @ngdoc service
         * @name tubularPopupService
         *
         * @description
         * Use `tubularPopupService` to show or generate popups with a `tbForm` inside.
         */
        .service('tubularPopupService', [
            '$modal', '$rootScope', 'tubularTemplateService',
            function tubularPopupService($modal, $rootScope, tubularTemplateService) {
                var me = this;

                me.onSuccessForm = function(callback) {
                    $rootScope.$on('tbForm_OnSuccessfulSave', callback);
                };

                me.onConnectionError = function(callback) {
                    $rootScope.$on('tbForm_OnConnectionError', callback);
                };

                /**
                 * Opens a new Popup
                 * @param {string} template 
                 * @param {object} model 
                 * @param {object} gridScope 
                 * @param {string} size 
                 * @returns {object} The Popup instance
                 */
                me.openDialog = function(template, model, gridScope, size) {
                    if (angular.isUndefined(template)) {
                        template = tubularTemplateService.generatePopup(model);
                    }

                    var dialog = $modal.open({
                        templateUrl: template,
                        backdropClass: 'fullHeight',
                        animation: false,
                        size: size,
                        controller: [
                            '$scope', function($scope) {
                                $scope.Model = model;

                                $scope.savePopup = function(innerModel) {
                                    innerModel = innerModel || $scope.Model;

                                    // If we have nothing to save and it's not a new record, just close
                                    if (!innerModel.$isNew && !innerModel.$hasChanges) {
                                        $scope.closePopup();
                                        return;
                                    }

                                    var result = innerModel.save();

                                    if (angular.isUndefined(result) || result === false) {
                                        return;
                                    }

                                    result.then(
                                        function(data) {
                                            $scope.$emit('tbForm_OnSuccessfulSave', data);
                                            $rootScope.$broadcast('tbForm_OnSuccessfulSave', data);
                                            $scope.Model.$isLoading = false;
                                            if (gridScope.autoRefresh) gridScope.retrieveData();
                                            dialog.close();
                                        }, function(error) {
                                            $scope.$emit('tbForm_OnConnectionError', error);
                                            $rootScope.$broadcast('tbForm_OnConnectionError', error);
                                            $scope.Model.$isLoading = false;
                                        });
                                };

                                $scope.closePopup = function() {
                                    if (angular.isDefined($scope.Model.revertChanges)) {
                                        $scope.Model.revertChanges();
                                    }

                                    dialog.close();
                                };
                            }
                        ]
                    });

                    return dialog;
                };
            }
        ])
        /**
         * @ngdoc service
         * @name tubularGridExportService
         *
         * @description
         * Use `tubularGridExportService` to export your `tbGrid` to CSV format.
         */
        .service('tubularGridExportService', function tubularGridExportService() {
            var me = this;

            me.getColumns = function(gridScope) {
                return gridScope.columns
                    .map(function(c) { return c.Name.replace(/([a-z])([A-Z])/g, '$1 $2'); });
            };

            me.getColumnsVisibility = function(gridScope) {
                return gridScope.columns
                    .map(function(c) { return c.Visible; });
            };

            me.exportAllGridToCsv = function(filename, gridScope) {
                var columns = me.getColumns(gridScope);
                var visibility = me.getColumnsVisibility(gridScope);

                gridScope.getFullDataSource(function(data) {
                    me.exportToCsv(filename, columns, data, visibility);
                });
            };

            me.exportGridToCsv = function(filename, gridScope) {
                var columns = me.getColumns(gridScope);
                var visibility = me.getColumnsVisibility(gridScope);

                gridScope.currentRequest = {};
                me.exportToCsv(filename, columns, gridScope.dataSource.Payload, visibility);
                gridScope.currentRequest = null;
            };

            me.exportToCsv = function(filename, header, rows, visibility) {
                var processRow = function(row) {
                    if (typeof (row) === 'object') {
                        row = Object.keys(row).map(function(key) { return row[key]; });
                    }

                    var finalVal = '';
                    for (var j = 0; j < row.length; j++) {
                        if (!visibility[j]) {
                            continue;
                        }

                        var innerValue = row[j] == null ? '' : row[j].toString();

                        if (row[j] instanceof Date) {
                            innerValue = row[j].toLocaleString();
                        }

                        var result = innerValue.replace(/"/g, '""');

                        if (result.search(/("|,|\n)/g) >= 0) {
                            result = '"' + result + '"';
                        }

                        if (j > 0) {
                            finalVal += ',';
                        }

                        finalVal += result;
                    }
                    return finalVal + '\n';
                };

                var csvFile = '';

                if (header.length > 0) {
                    csvFile += processRow(header);
                }

                for (var i = 0; i < rows.length; i++) {
                    csvFile += processRow(rows[i]);
                }

                // Add "\uFEFF" (UTF-8 BOM)
                var blob = new Blob(["\uFEFF" + csvFile], { type: 'text/csv;charset=utf-8;' });
                window.saveAs(blob, filename);
            };
        })
        /**
         * @ngdoc service
         * @name tubularGridFilterService
         *
         * @description
         * The `tubularGridFilterService` service is a internal helper to setup any `FilterModel` with a UI.
         */
        .service('tubularGridFilterService', [
            'tubulargGridFilterModel', '$compile', '$filter', function tubularGridFilterService(FilterModel, $compile, $filter) {
                var me = this;

                me.applyFilterFuncs = function(scope, el, attributes, openCallback) {
                    scope.$component = scope.$parent.$component;
                    
                    scope.$watch('filter.Operator', function(val) {
                        if (val === 'None') scope.filter.Text = '';
                    });

                    scope.retrieveData = function() {
                        var columns = scope.$component.columns.filter(function(el) {
                            return el.Name === scope.filter.Name;
                        });

                        if (columns.length !== 0) {
                            columns[0].Filter = scope.filter;
                        }

                        scope.$component.retrieveData();
                        scope.close();
                    };

                    scope.clearFilter = function() {
                        if (scope.filter.Operator != 'Multiple') {
                            scope.filter.Operator = 'None';
                        }

                        scope.filter.Text = '';
                        scope.filter.Argument = [];
                        scope.filter.HasFilter = false;
                        scope.retrieveData();
                    };

                    scope.applyFilter = function() {
                        scope.filter.HasFilter = true;
                        scope.retrieveData();
                    };

                    scope.close = function() {
                        $(el).find('.btn-popover').popover('hide');
                    };

                    scope.open = function() {
                        $(el).find('.btn-popover').popover('toggle');
                    };

                    scope.checkEvent = function(keyEvent) {
                        if (keyEvent.which === 13) {
                            scope.applyFilter();
                            keyEvent.preventDefault();
                        }
                    };

                    $(el).find('.btn-popover').popover({
                        html: true,
                        placement: 'bottom',
                        trigger: 'manual',
                        content: function() {
                            var selectEl = $(this).next().find('select').find('option').remove().end();
                            angular.forEach(scope.filterOperators, function(val, key) {
                                $(selectEl).append('<option value="' + key + '">' + val + '</option>');
                            });

                            return $compile($(this).next().html())(scope);
                        }
                    });

                    $(el).find('.btn-popover').on('show.bs.popover', function(e) {
                        $('.btn-popover').not(e.target).popover("hide");
                    });

                    if (angular.isDefined(openCallback)) {
                        $(el).find('.btn-popover').on('shown.bs.popover', openCallback);
                    }
                };

                /**
                 * Creates a `FilterModel` using a scope and an Attributes array
                 */
                me.createFilterModel = function(scope, lAttrs) {
                    scope.filter = new FilterModel(lAttrs);
                    scope.filter.Name = scope.$parent.column.Name;
                    var columns = scope.$component.columns.filter(function(el) {
                        return el.Name === scope.filter.Name;
                    });

                    if (columns.length === 0) return;

                    scope.$watch('filter', function(n) {
                        if (columns[0].Filter.Text != n.Text) {
                            n.Text = columns[0].Filter.Text;

                            if (columns[0].Filter.Operator != n.Operator) {
                                n.Operator = columns[0].Filter.Operator;
                            }
                        }

                        scope.filter.HasFilter = columns[0].Filter.HasFilter;
                    });

                    columns[0].Filter = scope.filter;
                    scope.dataType = columns[0].DataType;
                    scope.filterOperators = columns[0].FilterOperators[scope.dataType];

                    if (scope.dataType === 'datetime' || scope.dataType === 'date') {
                        scope.filter.Argument = [new Date()];
                        scope.filter.Operator = 'Equals';
                    }

                    if (scope.dataType === 'numeric' || scope.dataType === 'boolean') {
                        scope.filter.Argument = [1];
                        scope.filter.Operator = 'Equals';
                    }

                    scope.filterTitle = lAttrs.title || $filter('translate')('CAPTION_FILTER');
                };
            }
        ])
        /**
         * @ngdoc service
         * @name tubularEditorService
         *
         * @description
         * The `tubularEditorService` service is a internal helper to setup any `TubularModel` with a UI.
         */
        .service('tubularEditorService', [
            '$filter', function tubularEditorService($filter) {
                var me = this;

                /*
                 * Returns the Default Scope parameters
                 */
                me.defaultScope = {
                    value: '=?',
                    isEditing: '=?',
                    editorType: '@',
                    showLabel: '=?',
                    label: '@?',
                    required: '=?',
                    format: '@?',
                    min: '=?',
                    max: '=?',
                    name: '@',
                    placeholder: '@?',
                    readOnly: '=?',
                    help: '@?',
                    defaultValue: '@?'
                };

                /**
                 * Setups a basic Date Editor Controller
                 * @param {string} format 
                 * @returns {array}  The controller definition
                 */
                me.dateEditorController = function(format) {
                    return [
                        '$scope', function(innerScope) {
                            innerScope.DataType = "date";

                            innerScope.$watch('value', function(val) {
                                if (typeof (val) === 'string') {
                                    innerScope.value = new Date(val);
                                }
                            });

                            innerScope.validate = function() {
                                if (angular.isDefined(innerScope.min)) {
                                    if (Object.prototype.toString.call(innerScope.min) !== "[object Date]") {
                                        innerScope.min = new Date(innerScope.min);
                                    }

                                    innerScope.$valid = innerScope.value >= innerScope.min;

                                    if (!innerScope.$valid) {
                                        innerScope.state.$errors = [$filter('translate')('EDITOR_MIN_NUMBER', $filter('date')(innerScope.min, innerScope.format))];
                                    }
                                }

                                if (!innerScope.$valid) {
                                    return;
                                }

                                if (angular.isDefined(innerScope.max)) {
                                    if (Object.prototype.toString.call(innerScope.max) !== "[object Date]") {
                                        innerScope.max = new Date(innerScope.min);
                                    }

                                    innerScope.$valid = innerScope.value <= innerScope.max;

                                    if (!innerScope.$valid) {
                                        innerScope.state.$errors = [$filter('translate')('EDITOR_MIN_NUMBER', $filter('date')(innerScope.min, innerScope.format))];
                                    }
                                }
                            };

                            me.setupScope(innerScope, format);
                        }
                    ];
                }

                /**
                 * Setups a new Editor, this functions is like a common class constructor to be used
                 * with all the tubularEditors.
                 */
                me.setupScope = function(scope, defaultFormat) {
                    scope.isEditing = angular.isUndefined(scope.isEditing) ? true : scope.isEditing;
                    scope.showLabel = scope.showLabel || false;
                    scope.label = scope.label || (scope.name || '').replace(/([a-z])([A-Z])/g, '$1 $2');
                    scope.required = scope.required || false;
                    scope.readOnly = scope.readOnly || false;
                    scope.format = scope.format || defaultFormat;
                    scope.$valid = true;

                    scope.checkValid = function() {
                        scope.$valid = true;
                        scope.state.$errors = [];

                        if ((angular.isUndefined(scope.value) && scope.required) ||
                        (Object.prototype.toString.call(scope.value) === "[object Date]" && isNaN(scope.value.getTime()) && scope.required)) {
                            scope.$valid = false;
                            scope.state.$errors = [$filter('translate')('EDITOR_REQUIRED')];

                            if (angular.isDefined(scope.$parent.Model)) {
                                scope.$parent.Model.$state[scope.Name] = scope.state;
                            }

                            return;
                        }

                        // Check if we have a validation function, otherwise return
                        if (angular.isUndefined(scope.validate)) {
                            return;
                        }

                        scope.validate();
                    };

                    // HACK: I need to know why
                    scope.$watch('label', function(n, o) {
                        if (angular.isUndefined(n)) {
                            scope.label = (scope.name || '').replace(/([a-z])([A-Z])/g, '$1 $2');
                        }
                    });

                    scope.$watch('value', function(newValue, oldValue) {
                        if (angular.isUndefined(oldValue) && angular.isUndefined(newValue)) {
                            return;
                        }

                        scope.state = {
                            $valid: function() {
                                scope.checkValid();
                                return this.$errors.length === 0;
                            },
                            $errors: []
                        };

                        scope.$valid = true;

                        // Try to match the model to the parent, if it exists
                        if (angular.isDefined(scope.$parent.Model)) {
                            if (angular.isDefined(scope.$parent.Model[scope.name])) {
                                scope.$parent.Model[scope.name] = newValue;

                                if (angular.isUndefined(scope.$parent.Model.$state)) {
                                    scope.$parent.Model.$state = [];
                                }

                                scope.$parent.Model.$state[scope.Name] = scope.state;
                            } else if (angular.isDefined(scope.$parent.Model.$addField)) {
                                scope.$parent.Model.$addField(scope.name, newValue);
                            }
                        }

                        scope.checkValid();
                    });

                    var parent = scope.$parent;

                    // We try to find a Tubular Form in the parents
                    while (true) {
                        if (parent == null) break;
                        if (angular.isDefined(parent.tubularDirective) &&
                        (parent.tubularDirective === 'tubular-form' ||
                            parent.tubularDirective === 'tubular-rowset')) {

                            if (scope.name === null) {
                                return;
                            }

                            if (parent.hasFieldsDefinitions !== false) {
                                throw 'Cannot define more fields. Field definitions have been sealed';
                            }

                            scope.$component = parent.tubularDirective === 'tubular-form' ? parent : parent.$component;

                            scope.Name = scope.name;

                            scope.bindScope = function() {
                                scope.$parent.Model = parent.model;

                                if (angular.equals(scope.value, parent.model[scope.Name]) === false) {
                                    if (angular.isDefined(parent.model[scope.Name])) {
                                        scope.value = (scope.DataType === 'date' && parent.model[scope.Name] != null) ?
                                            new Date(parent.model[scope.Name]) :
                                            parent.model[scope.Name];
                                    }

                                    parent.$watch(function() {
                                        return scope.value;
                                    }, function(value) {
                                        parent.model[scope.Name] = value;
                                    });
                                }

                                if ((!scope.value || scope.value == null) && (scope.defaultValue && scope.defaultValue != null)) {
                                    if (scope.DataType === 'date' && scope.defaultValue != null) {
                                        scope.defaultValue = new Date(scope.defaultValue);
                                    }
                                    if (scope.DataType === 'numeric' && scope.defaultValue != null) {
                                        scope.defaultValue = parseFloat(scope.defaultValue);
                                    }

                                    scope.value = scope.defaultValue;
                                }

                                if (angular.isUndefined(parent.model.$state)) {
                                    parent.model.$state = {};
                                }

                                parent.model.$state[scope.Name] = {
                                    $valid: function() {
                                        scope.checkValid();
                                        return this.$errors.length === 0;
                                    },
                                    $errors: []
                                };

                                if (angular.equals(scope.state, parent.model.$state[scope.Name]) === false) {
                                    scope.state = parent.model.$state[scope.Name];
                                }
                            };

                            parent.fields.push(scope);

                            break;
                        }

                        parent = parent.$parent;
                    }
                };
            }
        ]);
})();